@@ -39,9 +39,9 @@ class qpwgraph_thumb::View : public QGraphicsView
 public:
 
 	// Constructor.
-	View(qpwgraph_canvas *canvas)
-		: QGraphicsView(canvas->viewport()),
-			m_canvas(canvas), m_drag_state(DragNone)
+	View(qpwgraph_thumb *thumb)
+		: QGraphicsView(thumb->canvas()->viewport()),
+			m_thumb(thumb), m_drag_state(DragNone)
 	{
 		QGraphicsView::setInteractive(false);
 
@@ -51,15 +51,16 @@ public:
 		QGraphicsView::setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 		QGraphicsView::setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
-		QPalette pal = m_canvas->palette();
+		qpwgraph_canvas *canvas = m_thumb->canvas();
+		QPalette pal = canvas->palette();
 		const QPalette::ColorRole role
-			= m_canvas->backgroundRole();
+			= canvas->backgroundRole();
 		const QColor& color = pal.color(role);
 		pal.setColor(role, color.darker(120));
 		QGraphicsView::setPalette(pal);
 		QGraphicsView::setBackgroundRole(role);
 
-		QGraphicsView::setScene(m_canvas->scene());
+		QGraphicsView::setScene(canvas->scene());
 	}
 
 protected:
@@ -67,11 +68,12 @@ protected:
 	// Compute the view(port) rectangle.
 	QRect viewRect() const
 	{
+		qpwgraph_canvas *canvas = m_thumb->canvas();
 		const QRect& vrect
-			= m_canvas->viewport()->rect();
+			= canvas->viewport()->rect();
 		const QRectF srect(
-			m_canvas->mapToScene(vrect.topLeft()),
-			m_canvas->mapToScene(vrect.bottomRight()));
+			canvas->mapToScene(vrect.topLeft()),
+			canvas->mapToScene(vrect.bottomRight()));
 		return QGraphicsView::viewport()->rect().intersected(QRect(
 			QGraphicsView::mapFromScene(srect.topLeft()),
 			QGraphicsView::mapFromScene(srect.bottomRight())))
@@ -136,7 +138,7 @@ protected:
 		}
 
 		if (m_drag_state == DragMove) {
-			m_canvas->centerOn(
+			m_thumb->canvas()->centerOn(
 				QGraphicsView::mapToScene(event->pos()));
 		}
 	}
@@ -146,7 +148,7 @@ protected:
 		QGraphicsView::mouseReleaseEvent(event);
 
 		if (m_drag_state != DragNone) {
-			m_canvas->centerOn(
+			m_thumb->canvas()->centerOn(
 				QGraphicsView::mapToScene(event->pos()));
 			m_drag_state = DragNone;
 		}
@@ -154,10 +156,15 @@ protected:
 
 	void wheelEvent(QWheelEvent *) {} // Ignore wheel events.
 
+	void contextMenuEvent(QContextMenuEvent *event)
+	{
+		m_thumb->contextMenu(event->globalPos());
+	}
+
 private:
 
 	// Instance members.
-	qpwgraph_canvas *m_canvas;
+	qpwgraph_thumb *m_thumb;
 
 	enum { DragNone = 0, DragStart, DragMove } m_drag_state;
 
@@ -172,7 +179,7 @@ private:
 qpwgraph_thumb::qpwgraph_thumb ( qpwgraph_canvas *canvas, Position position )
 	: QFrame(canvas), m_canvas(canvas), m_position(position), m_view(nullptr)
 {
-	m_view = new View(m_canvas);
+	m_view = new View(this);
 
 	QVBoxLayout *layout = new QVBoxLayout();
 	layout->setSpacing(0);
@@ -217,6 +224,13 @@ qpwgraph_thumb::Position qpwgraph_thumb::position (void) const
 }
 
 
+// Emit context-menu request.
+void qpwgraph_thumb::contextMenu ( const QPoint& pos )
+{
+	emit contextMenuRequested(pos);
+}
+
+
 // Update view.
 void qpwgraph_thumb::updatePosition (void)
 {
